from viggocore.common.subsystem import operation
from viggocore.common.subsystem.pagination import Pagination
from viggocore.common import manager

from viggofiscal.subsystem.financeiro.centro_resultado.resource \
    import CentroResultado


class List(operation.List):

    def do(self, session, **kwargs):
        id = kwargs.pop('id', None)

        query = session.query(CentroResultado)

        query = self.manager.apply_filters(query, CentroResultado, **kwargs)
        if id is not None:
            query = query.filter(CentroResultado.id != id)
        query = query.distinct()

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(CentroResultado, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(CentroResultado)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.list = List(self)



from sqlalchemy import UniqueConstraint, orm

from viggocore.database import db
from viggocore.common.subsystem import entity


class CodigoSegurancaContribuinte(entity.Entity, db.Model):

    attributes = ['domain_org_id', 'id_csc', 'csc']
    attributes += entity.Entity.attributes

    domain_org_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain_org.id'), nullable=False)
    id_csc = db.Column(db.Numeric(10), nullable=False)
    csc = db.Column(db.String(36), nullable=False)

    domain_org = orm.relationship(
        'DomainOrg',
        backref=orm.backref('codigo_seguranca_contribuinte_domain_org'))

    __table_args__ = (
        UniqueConstraint(
            'domain_org_id', 'id_csc', 'csc',
            name='codigo_seguranca_contribuinte_domain_org_id_csc_csc_uk'),)

    def __init__(self, id, domain_org_id, id_csc, csc,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_org_id = domain_org_id
        self.id_csc = id_csc
        self.csc = csc

    @classmethod
    def individual(cls):
        return 'codigo_seguranca_contribuinte'



from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt


class BaseReduzidaIcmsST():

    def __init__(self, base_icms_proprio: float, mva: float,
                 percentual_reducao_st: float,
                 valor_ipi: float = 0.0):
        self.base_icms_proprio = float(base_icms_proprio)
        self.mva = float(mva)
        self.percentual_reducao_st = float(percentual_reducao_st)
        self.valor_ipi = float(valor_ipi)

    def calcular_base_reduzida_icms_st(self) -> float:
        base_icms_st = self.base_icms_proprio * (1 + (self.mva / 100))
        base_icms_st = (
            base_icms_st - (base_icms_st * (self.percentual_reducao_st / 100)))
        base_reduzida_icms_st = base_icms_st + self.valor_ipi
        return round_abnt(base_reduzida_icms_st, 2)



from enum import Enum

import sqlalchemy
from sqlalchemy import orm
from viggocore.common.subsystem import entity
from viggocore.database import db


class NaturezaFinanceiraTipo(Enum):
    RECEITA = 'RECEITA'
    DESPESA = 'DESPESA'


class NaturezaFinanceira(entity.Entity, db.Model):

    attributes = ['domain_id', 'natureza_financeira_id', 'descricao',
                  'tipo', 'periodicidade']
    attributes += entity.Entity.attributes

    domain_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain.id'), nullable=False)

    natureza_financeira_id = db.Column(
        db.CHAR(32), db.ForeignKey('natureza_financeira.id'), nullable=True)

    domain = orm.relationship(
        'Domain', backref=orm.backref('natureza_financeira_domain'))

    natureza_financeira = orm.relationship(
        'NaturezaFinanceira', remote_side='NaturezaFinanceira.id',
        backref=orm.backref('natureza_financeira_natureza_financeira'))

    descricao = db.Column(db.String(255), nullable=False)
    tipo = db.Column(sqlalchemy.Enum(NaturezaFinanceiraTipo), nullable=False)
    periodicidade = db.Column(db.Boolean(), nullable=False)

    def __init__(self, id, domain_id, descricao, tipo, periodicidade,
                 natureza_financeira_id=None,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_id = domain_id
        self.descricao = descricao
        self.tipo = tipo
        self.periodicidade = periodicidade
        self.natureza_financeira_id = natureza_financeira_id

    def is_stable(self):
        return self.natureza_financeira_id != self.id and super().is_stable()

    @classmethod
    def individual(cls):
        return 'natureza_financeira'



from enum import Enum, IntEnum
import sqlalchemy
from sqlalchemy import orm
from viggocore.database import db
from viggocore.common.subsystem import entity
from viggocore.common import exception


class TipoIpi(Enum):
    AD_VALOREM = 'AD_VALOREM'
    ESPECIFICA = 'ESPECIFICA'


class RegraFiscalModelo(IntEnum):
    NFE = 55
    NFCE = 65
    NFE_NFCE = 5565


class RegraFiscal(entity.Entity, db.Model):

    attributes = ['domain_org_id', 'cstcofins_id', 'csticms_id', 'cstpis_id',
                  'cfop_id', 'cstipi_id', 'descricao', 'aliq_especifica_icms',
                  'aliq_especifica_icms_st', 'aliq_reducao_base_icms',
                  'aliq_reducao_base_st', 'aliq_diferimento', 'indice_difal',
                  'difal_base_dupla', 'icms_sob_pis_cofins', 'icms_sob_frete',
                  'tipo_ipi', 'icms_sob_ipi', 'codigo_cst_icms',
                  'codigo_cst_pis', 'codigo_cfop', 'codigo_cst_ipi',
                  'codigo_cst_cofins', 'mod_bc', 'mod_bc_st',
                  'nao_gera_receita',
                  'nao_gera_estoque', 'cst_icms_trabalha_st', 'modelo',
                  'numero_modelo']
    attributes += entity.Entity.attributes

    domain_org_id = db.Column(
        db.CHAR(32), db.ForeignKey('domain_org.id'), nullable=False)
    cstcofins_id = db.Column(
        db.CHAR(32), db.ForeignKey('cstcofins.id'), nullable=False)
    cstcofins = orm.relationship('Cstcofins', backref=orm.backref('cstcofinss'))
    csticms_id = db.Column(
        db.CHAR(32), db.ForeignKey('csticms.id'), nullable=False)
    csticms = orm.relationship('Csticms', backref=orm.backref('csticmss'))
    cstpis_id = db.Column(
        db.CHAR(32), db.ForeignKey('cstpis.id'), nullable=False)
    cstpis = orm.relationship('Cstpis', backref=orm.backref('cstpiss'))
    cfop_id = db.Column(
        db.CHAR(32), db.ForeignKey('cfop.id'), nullable=False)
    cfop = orm.relationship('Cfop', backref=orm.backref('cfops'))
    cstipi_id = db.Column(
        db.CHAR(32), db.ForeignKey('cstipi.id'), nullable=True)
    cstipi = orm.relationship('Cstipi', backref=orm.backref('cstipis'))

    descricao = db.Column(db.String(100), nullable=False)
    aliq_especifica_icms = db.Column(
        db.Numeric(5, 2), nullable=False, default=0, server_default="0")
    aliq_especifica_icms_st = db.Column(
        db.Numeric(5, 2), nullable=False, default=0, server_default="0")
    aliq_reducao_base_icms = db.Column(
        db.Numeric(5, 2), nullable=False, default=0, server_default="0")
    aliq_reducao_base_st = db.Column(
        db.Numeric(5, 2), nullable=False, default=0, server_default="0")
    aliq_diferimento = db.Column(
        db.Numeric(5, 2), nullable=False, default=0, server_default="0")
    indice_difal = db.Column(db.Boolean, nullable=False)
    difal_base_dupla = db.Column(db.Boolean, nullable=False)
    icms_sob_pis_cofins = db.Column(db.Boolean, nullable=False)
    icms_sob_frete = db.Column(db.Boolean, nullable=False)
    tipo_ipi = db.Column(sqlalchemy.Enum(TipoIpi), nullable=True)

    icms_sob_ipi = db.Column(db.Boolean, nullable=True)

    mod_bc = db.Column(db.Numeric(1), nullable=True)
    mod_bc_st = db.Column(db.Numeric(1), nullable=True)

    nao_gera_receita = db.Column(
        db.Boolean, nullable=False, default=False, server_default='false')
    nao_gera_estoque = db.Column(
        db.Boolean, nullable=False, default=False, server_default='false')

    modelo = db.Column(sqlalchemy.Enum(RegraFiscalModelo),
                       nullable=False, server_default='NFE')

    def __init__(self, id, domain_org_id, cstcofins_id, csticms_id,
                 cfop_id, cstipi_id, descricao, indice_difal, difal_base_dupla,
                 icms_sob_pis_cofins, icms_sob_frete, modelo,
                 aliq_especifica_icms=0, aliq_especifica_icms_st=0,
                 aliq_reducao_base_icms=0, aliq_reducao_base_st=0,
                 aliq_diferimento=0, cstpis_id=None, tipo_ipi=None,
                 icms_sob_ipi=None, mod_bc=None, mod_bc_st=None,
                 nao_gera_receita=False, nao_gera_estoque=False,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.domain_org_id = domain_org_id
        self.cstcofins_id = cstcofins_id
        self.csticms_id = csticms_id
        self.cfop_id = cfop_id
        self.cstipi_id = cstipi_id
        self.descricao = descricao
        self.aliq_especifica_icms = float(aliq_especifica_icms)
        self.aliq_especifica_icms_st = float(aliq_especifica_icms_st)
        self.aliq_reducao_base_icms = float(aliq_reducao_base_icms)
        self.aliq_reducao_base_st = float(aliq_reducao_base_st)
        self.indice_difal = indice_difal
        self.difal_base_dupla = difal_base_dupla
        self.icms_sob_pis_cofins = icms_sob_pis_cofins
        self.icms_sob_frete = icms_sob_frete
        self.cstpis_id = cstpis_id
        self.tipo_ipi = tipo_ipi
        self.icms_sob_ipi = icms_sob_ipi
        self.aliq_diferimento = float(aliq_diferimento)
        self.mod_bc = mod_bc
        self.mod_bc_st = mod_bc_st
        self.nao_gera_receita = nao_gera_receita
        self.nao_gera_estoque = nao_gera_estoque
        self.modelo = modelo

    def is_stable(self):
        # validação se icms_sob_ipi
        if (self.icms_sob_ipi is True and
           (self.tipo_ipi is None and self.cstipi_id is None)):
            raise exception.BadRequest(
                'Se icms_sob_ipi é verdadeiro então ' +
                'tipo_ipi e cstipi_id devem ser informados.')

        # TODO(JorgeSilva): verificar como fazer essas validações do tipo de
        # icms pois na criação eu ainda n tenho os backrefs.
        # # validações por tipo de icms
        # if self.csticms.cstcsosn == 20 and self.aliq_reducao_base_icms <= 0:
        #     raise exception.BadRequest('Se o cst_icms for igual a 20 a ' +
        #         'aliq_reducao_base_icms tem que ser maior que zero.')
        # elif self.csticms.cstcsosn == 51 and self.aliq_diferimento <= 0:
        #     raise exception.BadRequest('Se o cst_icms for igual a 51 a ' +
        #         'aliq_diferimento tem que ser maior que zero.')
        return True

    def informativo(self):
        if self.codigo_cst_icms in ['40', '41', '50', '60', '102', '103',
                                    '400', '500']:
            return True
        return False

    @property
    def codigo_cst_icms(self):
        return self.csticms.cstcsosn

    @property
    def codigo_cst_cofins(self):
        return self.cstcofins.cst

    @property
    def codigo_cst_pis(self):
        return self.cstpis.cst

    @property
    def codigo_cfop(self):
        return self.cfop.cfop

    @property
    def codigo_cst_ipi(self):
        if self.cstipi is None:
            return None
        return self.cstipi.cst

    @property
    def cst_icms_trabalha_st(self):
        return self.csticms.trabalha_st

    @property
    def numero_modelo(self):
        return self.modelo.value

    @classmethod
    def individual(cls):
        return 'regra_fiscal'



from viggocore.common.subsystem import operation, manager
from viggocore.common import exception


class Create(operation.Create):

    def post(self):

        natureza_list = ['Compra', 'Venda', 'Transferência', 'Devolução',
                         'Importação', 'Consignação', 'Outra',
                         'Remessa para fins de demonstração',
                         'Remessa para fins de industrialização']

        domain_org_id = self.entity.id
        if domain_org_id is not None:
            for item in natureza_list:
                natureza = {
                    'domain_org_id': domain_org_id,
                    'titulo': item,
                    'descricao': item
                }
                self.manager.api.natureza_operacaos().create(**natureza)


class UpdateSettings(operation.Update):

    def pre(self, session, id: str, **kwargs) -> bool:
        self.settings = kwargs
        if self.settings is None or not self.settings:
            raise exception.BadRequest("Erro! There is not a setting")
        return super().pre(session=session, id=id)

    def do(self, session, **kwargs):
        result = {}
        for key, value in self.settings.items():
            new_value = self.entity.update_setting(key, value)
            result[key] = new_value
        super().do(session)

        return result


class RemoveSettings(operation.Update):

    def pre(self, session, id: str, **kwargs) -> bool:
        self.keys = kwargs.get('keys', [])
        if not self.keys:
            raise exception.BadRequest('Erro! keys are empty')
        super().pre(session, id=id)

        return self.entity.is_stable()

    def do(self, session, **kwargs):
        result = {}
        for key in self.keys:
            value = self.entity.remove_setting(key)
            result[key] = value
        super().do(session=session)

        return result


class GetDomainOrgSettingsByKeys(operation.Get):

    def pre(self, session, id, **kwargs):
        self.keys = kwargs.get('keys', [])
        if not self.keys:
            raise exception.BadRequest('Erro! keys are empty')
        return super().pre(session, id=id)

    def do(self, session, **kwargs):
        entity = super().do(session=session)
        settings = {}
        for key in self.keys:
            value = entity.settings.get(key, None)
            if value is not None:
                settings[key] = value
        return settings


class Manager(manager.Manager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.create = Create(self)
        self.update_settings = UpdateSettings(self)
        self.remove_settings = RemoveSettings(self)
        self.get_domain_org_settings_by_keys = GetDomainOrgSettingsByKeys(self)



from viggocore.common import subsystem
from viggofiscal.subsystem.parametrizacao.csticms \
    import resource, manager, controller

subsystem = subsystem.Subsystem(resource=resource.Csticms,
                                manager=manager.Manager,
                                controller=controller.Controller)



from viggocore.common import manager
from viggocore.common.subsystem import operation
from viggocore.common.subsystem.pagination import Pagination

from viggofiscal.subsystem.parametrizacao.regra_fiscal.resource import \
    RegraFiscal, RegraFiscalModelo


class List(operation.List):

    def do(self, session, **kwargs):
        # pega o campo status como uma lista de status separado por vírgula
        modelo = kwargs.pop('modelo', '')

        query = session.query(RegraFiscal)
        query = self.manager.apply_filters(query, RegraFiscal, **kwargs)

        # aplica a filtragem pela lista de status
        if len(modelo) > 0:
            modelo_list = str(modelo).split(',')
            modelo_list_enum = [RegraFiscalModelo[ml] for ml in modelo_list]
            query = query.filter(RegraFiscal.modelo.in_(modelo_list_enum))

        dict_compare = {}
        query = self.manager.apply_filters_includes(
            query, dict_compare, **kwargs)

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(RegraFiscal, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(RegraFiscal)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.list = List(self)



from viggocore.common import exception
from viggocore.common.subsystem import manager, operation


class LimparPorSigla(operation.Delete):

    def pre(self, session, **kwargs):
        self.sigla = kwargs.pop('sigla', None)
        if self.sigla is None:
            raise exception.BadRequest('A sigla é obrigatória.')
        return True

    def do(self, session, **kwargs):
        query = """
            DELETE FROM ncm_ibpt AS ni
            WHERE ni.uf = \'{sigla}\';

            DELETE FROM ncm_ibpt_erro AS nie
            WHERE nie.uf = \'{sigla}\';
        """
        query = query.format(sigla=self.sigla)
        session.execute(query)
        return True


class Manager(manager.Manager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.limpar_por_sigla = LimparPorSigla(self)



from viggocore.common.subsystem import router


class Router(router.Router):

    def __init__(self, collection, routes=[]):
        super().__init__(collection, routes)

    @property
    def routes(self):
        return super().routes + [
            {
                'action': 'Analisar a criação ou atualiza',
                'method': 'POST',
                'url': self.collection_url + '/analisar',
                'callback': 'analisar'
            },
        ]



from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt


class PisCofins03():

    def __init__(self, base_calculo: float,
                 aliq_por_unidade_pis: float):
        # A Base de PIS será a Quantidade (qTrib) do produto na operação
        self.base_calculo = float(base_calculo)
        # Valor por Unidade Tributável
        self.aliq_por_unidade_pis = float(aliq_por_unidade_pis)

    def valor_pis(self):
        valor_pis = (self.aliq_por_unidade_pis * self.base_calculo)
        return round_abnt(valor_pis, 2)



from sqlalchemy import and_
from viggocore.common import manager
from viggocore.common.subsystem import operation
from viggocore.subsystem.user.resource import User
from viggofiscal.subsystem.parametrizacao.terminal.resource import (
    Terminal, TerminalOperador)
from viggocore.common.subsystem.pagination import Pagination


class Create(operation.Create):

    def do(self, session, **kwargs):
        super().do(session=session, **kwargs)

        self.manager.remover_users_de_outros_terminais(
            terminal=self.entity, session=session)

        return self.entity


class Update(operation.Update):

    def do(self, session, **kwargs):
        self.entity = super().do(session=session, **kwargs)

        self.manager.remover_users_de_outros_terminais(
            terminal=self.entity, session=session)

        return self.entity


class Analisar(operation.Create):

    def pre(self, session, **kwargs):
        return True

    def do(self, session, **kwargs):
        terminal_id = kwargs.get('id', '')
        operadores = kwargs.get('operadores', [])
        operadores_em_uso = []
        if len(operadores) > 0:
            operadores_em_uso = self.manager.get_operadores_em_uso(
                session, terminal_id, operadores)
        return operadores_em_uso


class List(operation.List):

    def do(self, session, **kwargs):
        query = session.query(Terminal)
        query = self.manager.apply_filters(query, Terminal, **kwargs)

        dict_compare = {}
        query = self.manager.apply_filters_includes(
            query, dict_compare, **kwargs)

        total_rows = None
        if self.manager.with_pagination(**kwargs):
            total_rows = query.count()

        pagination = Pagination.get_pagination(Terminal, **kwargs)
        if pagination.order_by is not None:
            pagination.adjust_order_by(Terminal)
        query = self.driver.apply_pagination(query, pagination)
        result = query.all()

        return (result, total_rows)


class Manager(manager.CommonManager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.create = Create(self)
        self.update = Update(self)
        self.analisar = Analisar(self)
        self.list = List(self)

    def remover_users_de_outros_terminais(self, terminal, session):
        if len(terminal.operadores) > 0:
            user_ids = terminal.get_user_id_dos_operadores()

            query = """
                DELETE FROM terminal_operador
                WHERE terminal_id <> \'{terminal_id}\' AND
                    user_id in {user_ids};
            """
            query = query.format(
                terminal_id=terminal.id,
                user_ids=str(user_ids).replace('[', '(').replace(']', ')'))
            session.execute(query)

    def get_operadores_em_uso(self, session, terminal_id, operadores):
        user_ids = [op.get('user_id', None) for op in operadores]
        query = session.query(User.name, Terminal.descricao). \
            join(TerminalOperador, TerminalOperador.user_id == User.id,
                 isouter=True). \
            join(Terminal, Terminal.id == TerminalOperador.terminal_id,
                 isouter=True). \
            filter(and_(TerminalOperador.terminal_id != terminal_id,
                        User.id.in_(user_ids)))
        query = query.distinct()
        result = query.all()

        return result



import os
import viggocore
import viggolocal

from flask_cors import CORS

from viggocore.system import System

from viggofiscal.subsystem.parametrizacao \
    import (portaria, uficms, forma_pagamento_sefaz, domain_org, cfop,
            cstcofins, csticms, cstipi, cstpis, origem, unidade_medida,
            ncm_ibpt, terminal,
            regra_fiscal, tipo_operacao, natureza_operacao, serial_fiscal,
            certificado_digital, ncm_ibpt_erro, codigo_seguranca_contribuinte,
            uficms_sugestao)
from viggofiscal.resources import SYSADMIN_EXCLUSIVE_POLICIES, \
    SYSADMIN_RESOURCES, USER_RESOURCES
from viggofiscal.subsystem.financeiro \
    import (portador, natureza_financeira, centro_resultado)


system = System('viggofiscal',
                [portaria.subsystem, uficms.subsystem,
                 forma_pagamento_sefaz.subsystem, domain_org.subsystem,
                 cfop.subsystem, cstcofins.subsystem, csticms.subsystem,
                 cstipi.subsystem, cstpis.subsystem, origem.subsystem,
                 unidade_medida.subsystem, ncm_ibpt.subsystem,
                 regra_fiscal.subsystem, tipo_operacao.subsystem,
                 natureza_operacao.subsystem, serial_fiscal.subsystem,
                 certificado_digital.subsystem, ncm_ibpt_erro.subsystem,
                 codigo_seguranca_contribuinte.subsystem,
                 terminal.subsystem, uficms_sugestao.subsystem,
                 portador.subsystem, natureza_financeira.subsystem,
                 centro_resultado.subsystem],
                USER_RESOURCES,
                SYSADMIN_RESOURCES,
                SYSADMIN_EXCLUSIVE_POLICIES)


class SystemFlask(viggocore.SystemFlask):

    def __init__(self):
        super().__init__(system, viggolocal.system)

    def configure(self):
        origins_urls = os.environ.get('ORIGINS_URLS', '*')
        CORS(self, resources={r'/*': {'origins': origins_urls}})

        self.config['BASEDIR'] = os.path.abspath(os.path.dirname(__file__))
        self.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = True
        viggofiscal_database_uri = os.getenv('VIGGOFISCAL_DATABASE_URI', None)
        if viggofiscal_database_uri is None:
            raise Exception(
                'VIGGOFISCAL_DATABASE_URI not defined in enviroment.')
        else:
            # URL os enviroment example for MySQL
            # export VIGGOFISCAL_DATABASE_URI=
            # mysql+pymysql://root:mysql@localhost:3306/viggofiscal
            self.config['SQLALCHEMY_DATABASE_URI'] = viggofiscal_database_uri



from viggocore.common import subsystem, controller
from viggofiscal.subsystem.financeiro.natureza_financeira \
    import resource, manager

subsystem = subsystem.Subsystem(resource=resource.NaturezaFinanceira,
                                manager=manager.Manager,
                                controller=controller.CommonController)



from viggocore.common import subsystem
from viggofiscal.subsystem.parametrizacao.portaria import resource, manager

subsystem = subsystem.Subsystem(resource=resource.Portaria,
                                manager=manager.Manager)



from viggofiscal.subsystem.calculo_fiscal.utils import round_abnt


class BaseReduzidaIcmsProprio():

    def __init__(self, valor_produto: float, valor_frete: float,
                 valor_seguro: float, despesas_acessorias: float,
                 valor_desconto: float, percentual_reducao: float,
                 valor_ipi: float = 0.0):
        self.valor_produto = float(valor_produto)
        self.valor_frete = float(valor_frete)
        self.valor_seguro = float(valor_seguro)
        self.despesas_acessorias = float(despesas_acessorias)
        self.valor_desconto = float(valor_desconto)
        self.percentual_reducao = float(percentual_reducao)
        self.valor_ipi = float(valor_ipi)

    def calcular_bc_icms(self) -> float:
        return (
            self.valor_produto + self.valor_frete + self.valor_seguro +
            self.despesas_acessorias - self.valor_desconto)

    def calcular_base_reduzida_icms_proprio(self) -> float:
        base_icms = self.calcular_bc_icms()
        base_reduzida_icms_proprio = (
            base_icms - (base_icms * (self.percentual_reducao / 100)) +
            self.valor_ipi)
        return round_abnt(base_reduzida_icms_proprio, 2)



from viggocore.common import subsystem
from viggofiscal.subsystem.parametrizacao.terminal \
    import resource, manager, controller, router

subsystem = subsystem.Subsystem(resource=resource.Terminal,
                                manager=manager.Manager,
                                controller=controller.Controller,
                                router=router.Router)





class BasePis():

    def __init__(self, valor_produto: float, valor_frete: float,
                 valor_seguro: float, despesas_acessorias: float,
                 valor_desconto: float, valor_icms: float):
        self.valor_produto = float(valor_produto)
        self.valor_frete = float(valor_frete)
        self.valor_seguro = float(valor_seguro)
        self.despesas_acessorias = float(despesas_acessorias)
        self.valor_desconto = float(valor_desconto)
        self.valor_icms = float(valor_icms)

    def calcular_base_pis(self):
        base_pis = (
            self.valor_produto + self.valor_frete + self.valor_seguro +
            self.despesas_acessorias - self.valor_desconto)
        base_pis -= self.valor_icms
        return base_pis



from viggocore.common import exception
from viggocore.common.subsystem import manager, operation


class Update(operation.Update):

    def pre(self, session, id, **kwargs):
        super().pre(session, id, **kwargs)

        active = kwargs.get('active', True)
        visivel_nota = kwargs.get('visivel_nota', True)

        if (active is False and self.entity.active is True
           and visivel_nota is True):
            raise exception.BadRequest(
                'Não é possível inativar uma portaria padrão para a nota.')
        return True


class Manager(manager.Manager):

    def __init__(self, driver):
        super(Manager, self).__init__(driver)
        self.update = Update(self)



from setuptools import setup, find_packages
from viggofiscal._version import version

REQUIRED_PACKAGES = [
    'viggocore>=1.0.0,<2.0.0',
    'viggolocal>=1.0.0',
    'flask-cors'
]

setup(
    name="viggofiscal",
    version=version,
    summary='ViggoFiscal Module Framework',
    description="ViggoFiscal Backend Flask REST service",
    packages=find_packages(exclude=["tests"]),
    install_requires=REQUIRED_PACKAGES
)
